@@ -553,6 +553,40 @@ class TestBaremetalNode(base.IronicTestCase):
 
         self.assert_calls()
 
+    def test_set_machine_power_on_with_retires(self):
+        # NOTE(TheJulia): This logic ends up testing power on/off and reboot
+        # as they all utilize the same helper method.
+        self.register_uris([
+            dict(
+                method='PUT',
+                status_code=503,
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid'],
+                            'states', 'power']),
+                validate=dict(json={'target': 'power on'})),
+            dict(
+                method='PUT',
+                status_code=409,
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid'],
+                            'states', 'power']),
+                validate=dict(json={'target': 'power on'})),
+            dict(
+                method='PUT',
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid'],
+                            'states', 'power']),
+                validate=dict(json={'target': 'power on'})),
+        ])
+        return_value = self.op_cloud.set_machine_power_on(
+            self.fake_baremetal_node['uuid'])
+        self.assertIsNone(return_value)
+
+        self.assert_calls()
+
     def test_set_machine_power_off(self):
         self.register_uris([
             dict(
@@ -630,6 +664,51 @@ class TestBaremetalNode(base.IronicTestCase):
 
         self.assert_calls()
 
+    def test_node_set_provision_state_with_retries(self):
+        deploy_node = self.fake_baremetal_node.copy()
+        deploy_node['provision_state'] = 'deploying'
+        active_node = self.fake_baremetal_node.copy()
+        active_node['provision_state'] = 'active'
+        self.register_uris([
+            dict(
+                method='PUT',
+                status_code=409,
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid'],
+                            'states', 'provision']),
+                validate=dict(json={'target': 'active',
+                                    'configdrive': 'http://host/file'})),
+            dict(
+                method='PUT',
+                status_code=503,
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid'],
+                            'states', 'provision']),
+                validate=dict(json={'target': 'active',
+                                    'configdrive': 'http://host/file'})),
+            dict(
+                method='PUT',
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid'],
+                            'states', 'provision']),
+                validate=dict(json={'target': 'active',
+                                    'configdrive': 'http://host/file'})),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     resource='nodes',
+                     append=[self.fake_baremetal_node['uuid']]),
+                 json=self.fake_baremetal_node),
+        ])
+        self.op_cloud.node_set_provision_state(
+            self.fake_baremetal_node['uuid'],
+            'active',
+            configdrive='http://host/file')
+
+        self.assert_calls()
+
     def test_node_set_provision_state_wait_timeout(self):
         deploy_node = self.fake_baremetal_node.copy()
         deploy_node['provision_state'] = 'deploying'
@@ -670,6 +749,7 @@ class TestBaremetalNode(base.IronicTestCase):
 
     def test_node_set_provision_state_wait_timeout_fails(self):
         # Intentionally time out.
+        self.fake_baremetal_node['provision_state'] = 'deploy wait'
         self.register_uris([
             dict(
                 method='PUT',
@@ -778,6 +858,55 @@ class TestBaremetalNode(base.IronicTestCase):
         self.assertEqual(available_node, return_value)
         self.assert_calls()
 
+    def test_wait_for_baremetal_node_lock_locked(self):
+        self.fake_baremetal_node['reservation'] = 'conductor0'
+        unlocked_node = self.fake_baremetal_node.copy()
+        unlocked_node['reservation'] = None
+        self.register_uris([
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     resource='nodes',
+                     append=[self.fake_baremetal_node['uuid']]),
+                 json=self.fake_baremetal_node),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     resource='nodes',
+                     append=[self.fake_baremetal_node['uuid']]),
+                 json=unlocked_node),
+        ])
+        self.assertIsNone(
+            self.op_cloud.wait_for_baremetal_node_lock(
+                self.fake_baremetal_node,
+                timeout=1))
+
+        self.assert_calls()
+
+    def test_wait_for_baremetal_node_lock_not_locked(self):
+        self.fake_baremetal_node['reservation'] = None
+        self.assertIsNone(
+            self.op_cloud.wait_for_baremetal_node_lock(
+                self.fake_baremetal_node,
+                timeout=1))
+
+        self.assertEqual(0, len(self.adapter.request_history))
+
+    def test_wait_for_baremetal_node_lock_timeout(self):
+        self.fake_baremetal_node['reservation'] = 'conductor0'
+        self.register_uris([
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     resource='nodes',
+                     append=[self.fake_baremetal_node['uuid']]),
+                 json=self.fake_baremetal_node),
+        ])
+        self.assertRaises(
+            exc.OpenStackCloudException,
+            self.op_cloud.wait_for_baremetal_node_lock,
+            self.fake_baremetal_node,
+            timeout=0.001)
+
+        self.assert_calls()
+
     def test_activate_node(self):
         self.fake_baremetal_node['provision_state'] = 'active'
         self.register_uris([
@@ -832,9 +961,7 @@ class TestBaremetalNode(base.IronicTestCase):
         node_uuid = self.fake_baremetal_node['uuid']
         # TODO(TheJulia): There is a lot of duplication
         # in testing creation. Surely this hsould be a helper
-        # or something. We should fix this, after we have
-        # ironicclient removed, as in the mean time visibility
-        # will be helpful.
+        # or something. We should fix this.
         node_to_post = {
             'chassis_uuid': None,
             'driver': None,
@@ -865,11 +992,10 @@ class TestBaremetalNode(base.IronicTestCase):
         self.assertDictEqual(self.fake_baremetal_node, return_value)
         self.assert_calls()
 
-    # TODO(TheJulia): After we remove ironicclient,
-    # we need to de-duplicate these tests. Possibly
-    # a dedicated class, although we should do it then
-    # as we may find differences that need to be accounted
-    # for newer microversions.
+    # TODO(TheJulia): We need to de-duplicate these tests.
+    # Possibly a dedicated class, although we should do it
+    # then as we may find differences that need to be
+    # accounted for newer microversions.
     def test_register_machine_enroll(self):
         mac_address = '00:01:02:03:04:05'
         nics = [{'mac': mac_address}]
@@ -1334,6 +1460,91 @@ class TestBaremetalNode(base.IronicTestCase):
 
         self.assert_calls()
 
+    def test_unregister_machine_locked_timeout(self):
+        mac_address = self.fake_baremetal_port['address']
+        nics = [{'mac': mac_address}]
+        self.fake_baremetal_node['provision_state'] = 'available'
+        self.fake_baremetal_node['reservation'] = 'conductor99'
+        self.register_uris([
+            dict(
+                method='GET',
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid']]),
+                json=self.fake_baremetal_node),
+            dict(
+                method='GET',
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid']]),
+                json=self.fake_baremetal_node),
+        ])
+        self.assertRaises(
+            exc.OpenStackCloudException,
+            self.op_cloud.unregister_machine,
+            nics,
+            self.fake_baremetal_node['uuid'],
+            timeout=0.001)
+        self.assert_calls()
+
+    def test_unregister_machine_retries(self):
+        mac_address = self.fake_baremetal_port['address']
+        nics = [{'mac': mac_address}]
+        port_uuid = self.fake_baremetal_port['uuid']
+        # NOTE(TheJulia): The two values below should be the same.
+        port_node_uuid = self.fake_baremetal_port['node_uuid']
+        port_url_address = 'detail?address=%s' % mac_address
+        self.fake_baremetal_node['provision_state'] = 'available'
+        self.register_uris([
+            dict(
+                method='GET',
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid']]),
+                json=self.fake_baremetal_node),
+            dict(
+                method='GET',
+                uri=self.get_mock_url(
+                    resource='ports',
+                    append=[port_url_address]),
+                json={'ports': [{'address': mac_address,
+                                 'node_uuid': port_node_uuid,
+                                 'uuid': port_uuid}]}),
+            dict(
+                method='DELETE',
+                status_code=503,
+                uri=self.get_mock_url(
+                    resource='ports',
+                    append=[self.fake_baremetal_port['uuid']])),
+            dict(
+                method='DELETE',
+                status_code=409,
+                uri=self.get_mock_url(
+                    resource='ports',
+                    append=[self.fake_baremetal_port['uuid']])),
+            dict(
+                method='DELETE',
+                uri=self.get_mock_url(
+                    resource='ports',
+                    append=[self.fake_baremetal_port['uuid']])),
+            dict(
+                method='DELETE',
+                status_code=409,
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid']])),
+            dict(
+                method='DELETE',
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid']])),
+        ])
+
+        self.op_cloud.unregister_machine(nics,
+                                         self.fake_baremetal_node['uuid'])
+
+        self.assert_calls()
+
     def test_unregister_machine_unavailable(self):
         # This is a list of invalid states that the method
         # should fail on.
